--- Main common api and loader
-- @author Oskar Eisemuth
-- @copyright 2017-2020
-- @module commonapi

--assert(commonapi ~= nil, "Common API 2 init should be loaded once into lua")
local commonapi = {
--- Version of Common API as integer: YYYYMMDD
	version = 20200627,
	dmp = function(...)
		local ok, inspect = pcall(require, "commonapi2.extlib.inspect")
		if (ok) then
			print(inspect(...))
		end
	end,
	_missing = {
		_native = true,
		ui = true,
	},
	_unique_threadid = tostring( {} ):sub(8),
	__doc__ = "",
	__members = {},
	__nested = {},

	_unique_threadid_check = function()
		local ourthreadid = tostring( {} ):sub(8)
		if (commonapi._unique_threadid == ourthreadid)  then
			print("commonapi2._unique_threadid_check() threadid match = "  ..ourthreadid)
		else
			print("commonapi2._unique_threadid_check() MISMATCH ".. commonapi._unique_threadid .. " !="..ourthreadid)
		end
	end,
}

local commonapi_versionsdetect = {
	steam_29416_1 = true,
	steam_29433_1 = true,
}


commonapi._reloadModule = function(key)
	print("commonapi2._reloadModule "..key)
	rawset(commonapi, key, nil)
	rawset(commonapi._missing, key, nil)
	package.loaded["commonapi2."..key] = nil
end

local commonapi_metatable = {
	__index = function(t, key)
		if (t._missing[key] ~= nil) then
			return nil
		end
		local ok, obj = pcall(require, "commonapi2."..key)
		if (ok) then
			t[key] = obj
			return t[key]
		end
		print("commonapi2."..key.." not found: "..obj)
		t._missing[key] = true
		--print(debug.traceback())
		return nil
	end
}

setmetatable(commonapi, commonapi_metatable)


local function detectOS()
	local libext = package.cpath:match("%.[\\|/]?%?%.(%a+)")
	local mypath = debug.getinfo(2, "S").source:sub(2)

	local libexttosys = {
		dll = 1,
		so = 2,
		dylib = 3
	}

	mypath = string.gsub(mypath, '\\', '/')
	mypath = string.gsub(mypath, "/res/scripts/commonapi2/init%.lua$", "/")

	local info = {
		sys = libexttosys[libext] or 0,
		libext = libext,
		path = mypath,
		settings = {}
	}

	commonapi._info = info
end


local function loadSettings()
	local result, luafnSettings, err
	local filename = commonapi._info.path.."settings.lua"
	local env = {}
	setmetatable(env, {__index=_G})
	luafnSettings, err = loadfile(filename, "bt", env)

	if (luafnSettings == nil) then
		return
	end

	result = luafnSettings()
	if (type(result) ~= "table") then
		print("commonapi2.init: Result of "..filename.. " not a table?!")
	end
	commonapi._info.settings = result
end

local function loadCommonAPILuaExtension(mystrings)
	local sys = commonapi._info.sys

	if not ( sys == 1 or sys == 2 ) then
	--if not ( sys == 1 ) then
		return
	end

	if (CommonAPILuaStateID ~= nil) then
		-- CommonAPI own lua state, no need to check
	else 
		if (getBuildVersion == nil or getBuildPrefix == nil) then
			print("commonapi2.init: getBuildVersion missing, please upgrade TPF2")
			return
		end
		local ugbuild = getBuildPrefix() .. '_'..getBuildVersion()..'_'..commonapi._info.sys


		if (type(commonapi._info.settings.buildoverwrite) == "string") then
			local buildoverwrite = commonapi._info.settings.buildoverwrite
			if (buildoverwrite ~= "") then
				commonapi_versionsdetect[buildoverwrite] = true
			end
		end

		if (commonapi_versionsdetect[ugbuild] ~= true) then
			local errormsg = "Your TPF2 version '" ..ugbuild.."' isn't listed as known good version, please update CommonAPI2"
			print("commonapi2.init: "..errormsg)

--[[
			local errorstr = "CommonAPI2:\nYour TPF2 version '" ..ugbuild.."' isn't listed as known good version"
			local eleid = "CommonAPI2.MainMenu.ErrorMessage"
			commonapi.dmp(_G)
			if (api and api.gui and api.gui.util and api.gui.comp and api.gui.comp.TextView) then
				if (api.gui.util.getById(eleid) == nil) then
					local eleui = api.gui.comp.TextView.new(errorstr)
					eleui.setId(eleid)
					local menuui = api.gui.util.getById("inGameConsole"):getParent():getParent():getParent():getParent()
					local menuuilayout = menuui:getLayout()
					menuuilayout.addItem(ele, 0, 1)
				end
			else
				print("Gui not found")
			end
--]]
			return
		end
	end

	local mypath = commonapi._info.path.."bin/?."..commonapi._info.libext
	local oldcpath = package.cpath
	local oldpath = package.path
	package.cpath = mypath
	package.path = ""

	commonapi._missing['_native'] = true
	commonapi._missing['ui'] = true
	local ok, dll = pcall(require, "CommonAPI2Native")
	package.cpath = oldcpath
	package.path = oldpath

	if not ok then
		print("commonapi2.init: Can't load native code CommonAPI2Native: "..dll)
		return
	elseif (type(dll) == 'table') then
		commonapi._native = dll
		if (type(commonapi._info.settings.verbosity) == "number") then
			commonapi._native.setDebugVerbosity(commonapi._info.settings.verbosity)
		end
		commonapi._native.setSettings(commonapi._info.settings)

		commonapi._native.doPatches()


		print("commonapi2.init: loadTranslationSystem", commonapi._native.getLanguage() )

		if ( commonapi._native.getGUIBindings ~= nil) then
			commonapi.ui = commonapi._nativeui
			commonapi.ui._activate()
		end

		if (type(commonapi._info.settings.uiscale) == "number") then
			commonapi._native.setUIScale(commonapi._info.settings.uiscale)
		end

		if (commonapi._events ~= nil) then
			if (commonapi._events._activate()) then
				commonapi.events = commonapi._events
			end
		end

--[[		if (commonapi._vehanimations ~= nil) then
			if (commonapi._vehanimations._activate()) then
				commonapi.vehanimations = commonapi._vehanimations
			end
		end
--]]



		commonapi._unique_threadid = commonapi._native.getLuaStatePtr()
		commonapi._unique_threadid_check = function()
			local ourthreadid = commonapi._native.getLuaStatePtr()
			if (commonapi._unique_threadid == ourthreadid)  then
				print("commonapi2._unique_threadid_check(n) threadid match = "  ..ourthreadid)
			else
				print("commonapi2._unique_threadid_check(native) MISMATCH ".. commonapi._unique_threadid .. " !="..ourthreadid)
			end
		end
	end
end

local function loadTranslationSystem(translations)
	if (setStrings == nil) then
		require "mod"
	end
	setStrings("eis_os_commonapi2_1", translations)

	if (getTextRes == nil or getTextRes == false) then
		getTextRes = function(s)
			return s
		end
	end

	local lang = "en"

	if (commonapi._native ~= nil) then
		lang = commonapi._native.getLanguage()
	end

	if (_ == nil) then
		local translatefn = 'function _(s) return translate("eis_os_commonapi2_1", "'..lang..'", s) end'
		load(translatefn)()
	end

	if (commonapi._native ~= nil) then
		if (translations[lang] ~= nil) then
			commonapi._native.setTranslations(translations[lang])
		else
			commonapi._native.setTranslations(translations["en"])
		end
	end
end

function commonapi._init(mystrings)
	if (_G['commonapi'] ~= nil) then
		local otherversion = _G['commonapi']
		if (otherversion.version ~= commonapi.version) then
			local twoversions = "\n\nERROR: CommonAPI2 with different version installed! You can only have one CommonAPI2 installed!\n\n"
			error("VERSIONS MISMACH(_init)" ..twoversions, 2)
		end
-- CommonAPI strings.lua could be loaded again in the same context
--		print("ERROR: Common API was loaded already? You can only have one commonapi installed!")
		return false
	end
	print("commonapi2.init", commonapi.version)
	_G['commonapi'] = commonapi

	detectOS()
	loadSettings()


	if (commonapi._info.settings.disablenative == true) then
		print("commonapi.init: Native code loading disabled by settings.lua")
	else
		loadCommonAPILuaExtension(mystrings)
	end

	loadTranslationSystem(mystrings)

	-- Need to load early for proper hooking...
	local mods = commonapi.mods
	return true
end


return commonapi