local modsettings = {}

local _log = commonapi.utils.logPrep("commonapi2.ui.modsettings")
local _utils = commonapi.utils
local _ = _utils._t
local ui = commonapi.ui;


local ModListWindow = require("commonapi2.ui.ModListWindow")
local ModDownloadWindow = require("commonapi2.ui.ModDownloadWindow")
local ModSettingsWindow = require("commonapi2.ui.ModSettingsWindow")



local serializeSettingsData = ModSettingsWindow.serializeSettingsData



--if (commonapi._native.paths.getUserPath())


function modsettings.createModListPath()
	local modlistpath = commonapi._info.path

	local userpath = commonapi._native.paths.getUserPath()
	if (userpath ~= nil) then
		modlistpath = userpath .. "modlists"
		commonapi._native.fs.mkdir(modlistpath)
	end
	return modlistpath
end

function modsettings.importActiveModsFileDialog()

	package.loaded["commonapi2.ui.FileDialog"] = nil
	local FileDialog = require("commonapi2.ui.FileDialog")
	
	local fd = FileDialog.createOpen()
	fd.id = "FileDialog_Modlist_Import"
	fd.caption = _("Modlist import")
	fd.path = modsettings.createModListPath()
	fd.fileext = ".modlist"
	fd.loadFile = function(self, filename)
		self:close()
		modsettings.importActiveMods(filename)
	end
	fd:open()
end


function modsettings.exportActiveModsFileDialog()
	package.loaded["commonapi2.ui.FileDialog"] = nil
	local FileDialog = require("commonapi2.ui.FileDialog")
	
	local fd = FileDialog.createSave()
	fd.id = "FileDialog_Modlist_Export"
	fd.caption = _("Modlist export")
	fd.path = modsettings.createModListPath()
	fd.fileext = ".modlist"
	fd.saveFile = function(self, filename)
		self:close()
		print(filename)
		modsettings.exportActiveMods(filename)
	end
	fd:open()
end

function modsettings.exportActiveMods(filename)
	local activeMods = {}

	local mods = commonapi.mods.getLoaded()
	local hstrings = {
		"--[[",
		"Created by CommonAPI2 "..commonapi.version,
		"",
	}

	if (#mods < 1) then
		ui.msgbox("modlist_exportempty", _("Modlist is empty"), _("Modlist export") )
		return
	end

	for modid, entry in pairs( mods ) do
		local data = entry["data"] or {}

		local tpfid = string.gsub(entry.id, "_(%d+)$", "")
		local activeMod = {
			tpfid,
			entry.version[1],
			modId = entry.id,
			version = entry.version
		}
		if (data.info ~= nil) then
			local info = data.info
			if (info.tfnetId ~= nil) then
				activeMod.tfnetId = info.tfnetId
			end
			if (info.steamId ~= nil) then
				activeMod.steamId = info.steamId
			end
		end
		activeMods[#activeMods+1] = activeMod

		hstrings[#hstrings+1] = entry.id.."\t"..entry.version[1].."."..entry.version[2].."\t("..entry.name..")"
	end
	hstrings[#hstrings+1] = ""
	hstrings[#hstrings+1] = "count: ".. #activeMods
	hstrings[#hstrings+1] = "--]]"

	local header = table.concat(hstrings, "\n")
	local serData = header.."\n\nreturn "..serializeSettingsData(activeMods, "").."\n"

	--local filename = commonapi._info.path .. "activemodlist.lua"
	local fh = io.open(filename, "w")
	if (fh) then
		fh:write(serData)
		fh:close()
	end
	_log("Exported active mod list to "..filename)

	ui.msgbox("modlist_exportok", "Exported active mod list to:\n"..filename, _("Modlist export") )
end





function modsettings.importActiveMods(filename)
	local env = {}
	local inmodlist
	--local filename = commonapi._info.path .. "activemodlist.lua"


	setmetatable(env, {__index=_G})
	luafnModList, err = loadfile(filename, "bt", env)
	if (luafnModList == nil) then
		ui.msgbox("modlist_importfailed", "FAILED: Imported from\n"..filename.."\nmissing file!", _("Modlist import") )
		return
	end

	inmodlist = luafnModList()
	if (type(inmodlist) ~= "table") then
		return
	end

	local _getByIdAny = commonapi.mods._getByIdAny
	local modlist = {}

	for i, entry in ipairs( inmodlist ) do
		local modid = entry[1]
		local version = entry[2]
		if (modid ~= nil and version ~= nil) then

			local modentry = _getByIdAny(modid.."_"..version)
			if (modentry == nil) then
				modentry = _getByIdAny(modid)
			end
			if (modentry ~= nil) then
--[[
				local tpfid = string.gsub(modentry.id, "_(%d+)$", "")
				local folder, fmodid = string.match(modentry.path, "([^/@]+)/([^/]+)/$")

				if (folder == "1066780") then
					tpfid = "*"..tpfid
				end
				if (folder == "staging_area") then
					tpfid = "?"..tpfid
				end
--]]
				local tpfid = string.gsub(modentry._internalmodid or modentry.id, "_(%d+)$", "")
				modlist[#modlist+1] = { tpfid, modentry.version[1] }
			end
		end
	end
	--commonapi.dmp(modlist)
	local result = commonapi._native.setActiveModList(modlist)
	
	if (result == -1) then
		ui.msgbox("modlist_importfailed", "Can't import without a mod list window!", _("Modlist import") )
		return
	end

	ui.msgbox("modlist_importok", "Imported from\n"..filename.." "..(result or ""), _("Modlist import") )
end




local function modsettings_ModListWindow_Open()
	ModListWindow.create()
end
local function modsettings_ModDownloadWindow_Open()
	ModDownloadWindow.create()
end


local function reloadcallback()
	package.loaded["commonapi2.ui.modsettings"] = nil
	package.loaded["commonapi2.ui.ModListWindow"] = nil
	package.loaded["commonapi2.ui.ModDownloadWindow"] = nil
	print("Reload callback called")
	local modsettings = require "commonapi2.ui.modsettings"
	modsettings.run()
end


function modsettings.run()
	ui.createModMenuEntry('modsettings_open', _("Mod settings"), modsettings_ModListWindow_Open)
	ui.createModMenuEntry('moddownload_open', _("Mod download"), modsettings_ModDownloadWindow_Open)


	ui.createModMenuEntry('modsettings_reload', "Reload Mod Settings", reloadcallback)
if (CommonAPILuaState == true) then
	ui.createModMenuEntry('modsettings_export', _("Modlist export"), modsettings.exportActiveModsFileDialog)
	ui.createModMenuEntry('modsettings_import', _("Modlist import"), modsettings.importActiveModsFileDialog)
end
	local modlisterrorwindow = require "commonapi2.ui.ModListErrorWindow"
	modlisterrorwindow.run()
end


function modsettings._showCommonAPISettings()
	_log("_showCommonAPISettings")

	local env = {
		data = function()
			return {}
		end,
		["_"] = commonapi.utils._t
	}

	local modentry = commonapi.mods._createModEntry("eis_os_commonapi2_1", commonapi._info.path, env)
	local win = ModSettingsWindow.createByModEntry(modentry)
	win:open()
end
return modsettings