local _utils = commonapi.utils
local _ = _utils._t

local ugglue = {}




local UIEvent = {}
function UIEvent.createClick(obj, item)
	if (item.onclick == nil) then
		return
	end
	if (obj._events == nil) then
		obj._events = {}
	end
	obj._events[#obj._events+1] = {
		item.onclick, item
	}
end

function UIEvent.createDblClick(obj, item)
	if (item.ondblclick == nil) then
		return
	end
	if (obj._events == nil) then
		obj._events = {}
	end
	obj._events[#obj._events+1] = {
		item.ondblclick, item
	}
end


function UIEvent.createChange(obj, item)
	if (item.onchange == nil) then
		return
	end
	if (obj._events == nil) then
		obj._events = {}
	end
	obj._events[#obj._events+1] = {
		item.onchange, item
	}
end

function UIEvent.fireEvents(obj)
	local events = obj._events
	if (events ~= nil and #events > 0) then
		commonapi.dmp(events)
		obj._events = {}
		for i, event in ipairs(events) do
			local fn = event[1]
			fn(event[2], obj)
		end
	end
end


local UIItemCreator = {

}
local UIItemRender = {
}

function UIItemCreator.Separator(self, item)
	local ele = api.gui.comp.Component.new("HorizontalLine")
	return ele
end

function UIItemCreator.Label(self, item)
	local ele = api.gui.comp.TextView.new(item.caption or "")
	--ele:setText(item.caption)
	return ele
end


function UIItemCreator.Text(self, item)
	local ele = api.gui.comp.TextView.new(item.caption or "")
	--ele:setText(item.caption)
	return ele
end

function UIItemCreator.Button(self, item)
	local eletext = api.gui.comp.TextView.new(item.caption or "Button")
	local ele = api.gui.comp.Button.new(eletext, true)

	local this = self
	local onclickfn = function()
		this.params[item.id] = true
		UIEvent.createClick(this, item)
	end
	ele:onClick(onclickfn)
	--ele:setId(item.id)
	return ele
end

function UIItemCreator.Checkbox(self, item)
	local param
	if (self.params[item.id] ~= nil) then
		param = self.params[item.id]
	else
		param = item.default or false
	end

	local ele = api.gui.comp.CheckBox.new(item.caption or "", "ui/checkbox0.tga", "ui/checkbox1.tga")
	--item.caption or "")
	--ele:setId(item.id)
	ele:setSelected(param, false)
	local this = self
	local onToggleFn = function(newbool)
		this.params[item.id] = newbool
		UIEvent.createClick(this, item)
	end
	ele:onToggle(onToggleFn)
	return ele
end


function UIItemCreator.TextInput(self, item)
	local changed = false
	local param = ""
	local value
	local ok
	
	if (self.params[item.id] ~= nil) then
		param = ""..self.params[item.id]
	else
		param = ""..item.default or ""
	end

	local ele = api.gui.comp.TextInputField.new(param)

	local this = self
	local onChangeFn = function(newvalue)
		if (newvalue == nil) then
			newvalue = ele:getText()
		end

		if (item.typ == "IntValue" or item.typ == "FloatValue") then
			this.params[item.id] = tonumber(newvalue)
		else
			this.params[item.id] = newvalue
		end
		UIEvent.createChange(this, item)
	end
	ele:onChange(onChangeFn)
	ele:onEnter(onChangeFn)

	return ele
end

UIItemCreator.IntValue = UIItemCreator.TextInput
UIItemCreator.FloatValue = UIItemCreator.TextInput

local UIWindow = {
	_open = true,

	id = "",
	uilist = {},
	params = {},
}

--- UI Window creation function
-- @tparam[opt] table o A table to use while construction
-- @treturn UIWindow
function UIWindow:new(o)
	local o = o or {}

	for k, v in pairs(self) do
		if (o[k] == nil) then
			o[k] = _utils.deepcopy(v)
		end
	end

	o._super = self
	return o
end

function UIWindow:getElementById(id)
	for k, item in pairs(self.uilist) do
		if (item.id == id) then
			return item
		end
	end
	return nil
end


function UIWindow:_render()
	self._ugidprefix = "XCoTPF2."..self.id.."."
	self._ugwindow = api.gui.util.getById(self._ugidprefix.."_window")
	self._ugwindowlayout = api.gui.util.getById(self._ugidprefix.."_layout")
	-- = api.gui.util.getById("XCoTPF2.ModSettingsWindow_eis_os_commonapi2_1._layout")
	print(self._ugidprefix)
	if (self._ugwindow == nil) then
		self._ugwindowlayout = api.gui.layout.BoxLayout.new("VERTICAL")

		self._ugwindowlayout:setId(self._ugidprefix.."_layout")

		--game.gui.window_create(self._ugidprefix.."_window", self.caption or "Test", self._ugidprefix.."_layout")
		self._ugwindow = api.gui.comp.Window.new( self.caption, self._ugwindowlayout )
		self._ugwindow:setId(self._ugidprefix.."_window")
		local that = self
		local closehandler = function()
			that._ugwindow:setVisible(false, false)
		end
		self._ugwindow:onClose(closehandler)

		--self._ugwindow = api.gui.util.getById(self._ugidprefix.."_window")
	end
	local this = self
	local stepfn = function()
		this:_fireevents()
	end
	self._ugwindow:onStep(stepfn)

	if (self._oldlist ~= self.uilist) then
		local numitems = self._ugwindowlayout:getNumItems()
		if (numitems > 0) then
			for i = numitems, 1, -1 do
				local tmp = self._ugwindowlayout:getItem(i-1)
				self._ugwindowlayout:removeItem(tmp)
			end
		end
		for k, item in pairs(self.uilist) do
			local typ = item.typ or "Label"
			local fn = UIItemCreator[typ]
			if (fn ~= nil) then
				local ele = fn(self, item)
				if (ele ~= false and ele ~= nil) then
					self._ugwindowlayout:addItem(ele)
				end
			end
		end
		self._oldlist = self.uilist
	end
end

function UIWindow:_fireevents()
	UIEvent.fireEvents(self)

	if (self.changed == true) then
		if (self.onchange ~= nil) then
			self:onchange()
		end
		self.changed = false
	end
end

--- Open this window
--
--
--
-- Warning: Proper id of window needs to be set before use!
function UIWindow:open()
	self._open = true
	self._firstframe = true
	self:_render()
	self._ugwindow:setVisible(true, false)
end

--- Close this window
function UIWindow:close()
	self._open = false
	self._ugelements = nil
	if (self._ugwindow ~= nil) then
		self._ugwindow:close()
	end
--	if (self._ugwindowlayout ~= nil) then
--		self._ugwindowlayout:destroy()
--		self._ugwindowlayout = nil
--	end
end

ugglue.Window = UIWindow

return ugglue